import joblib
from joblib.compressor import CompressorWrapper

from .file import ZstandardFile

DEFAULT_COMPRESS_LEVELS = (1, 2, 3, 6, 9, 12, 15, 18, 20, 22)


class ZStandardCompressorWrapper(CompressorWrapper):
    """joblib CompressorWrapper for Zstandard format"""

    def __init__(self, compress_levels, compressor_args, decompressor_args):
        super().__init__(ZstandardFile, b"\x28\xb5\x2f\xfd", ".zst")
        self._comprss_levels = compress_levels or DEFAULT_COMPRESS_LEVELS
        self._compressor_args = compressor_args
        self._decompressor_args = decompressor_args

    def compressor_file(self, fileobj, compresslevel=None):
        """Returns an instance of a compressor file object."""
        return self._factory(fileobj, "w", compresslevel)

    def decompressor_file(self, fileobj):
        """Returns an instance of a decompressor file object."""
        return self._factory(fileobj, "r")

    def _factory(self, fileobj, mode, compresslevel=None):
        if compresslevel is None:
            compresslevel = 2
        level = self._comprss_levels[compresslevel]
        return self.fileobj_factory(
            fileobj,
            mode=mode,
            compresslevel=level,
            compressor_args=self._compressor_args,
            decompressor_args=self._decompressor_args,
        )


def register(compress_levels=None, compressor_args=None, decompressor_args=None, force=True):
    if compress_levels and (not isinstance(compress_levels, (list, tuple)) or len(compress_levels) != 10):
        raise ValueError("compress_levels L must be list or tuple, len(L) == 10, 1 <= L[i] <= 22.")
    joblib.register_compressor(
        "zstd", ZStandardCompressorWrapper(compress_levels, compressor_args, decompressor_args), force=force,
    )



import io
import pathlib

import zstandard as zstd


class ZstandardFile(io.RawIOBase):
    """File like object interface of Zstanrard archive"""

    def __init__(
        self, filename_or_obj, mode="r", compresslevel=3, compressor_args=None, decompressor_args=None,
    ):
        super().__init__()
        self.ctx = None
        self.stream = None
        self.fileobj = None
        if mode == "w":
            if isinstance(filename_or_obj, (str, pathlib.Path)):
                self.fileobj = open(filename_or_obj, "wb")
            else:
                self.fileobj = filename_or_obj
            if not compressor_args:
                compressor_args = {}
            self.ctx = zstd.ZstdCompressor(level=compresslevel, **compressor_args)
            self.stream = self.ctx.stream_writer(self.fileobj)
        else:
            if isinstance(filename_or_obj, (str, pathlib.Path)):
                self.fileobj = open(filename_or_obj, "rb")
            else:
                self.fileobj = filename_or_obj
            if not decompressor_args:
                decompressor_args = {}
            self.ctx = zstd.ZstdDecompressor(**decompressor_args)
            self.stream = self.ctx.stream_reader(self.fileobj)

    def close(self):
        self.stream.close()
        self.fileobj.close()

    @property
    def closed(self):
        return self.fileobj.closed

    def flush(self):
        return self.stream.flush()

    def writable(self):
        return self.fileobj.writable()

    def readable(self):
        return self.fileobj.readable()

    def seekable(self):
        return self.stream.seekable()

    def tell(self, *args, **kwargs):
        return self.stream.tell(*args, **kwargs)

    def seek(self, *args, **kwargs):
        return self.stream.seek(*args, **kwargs)

    def truncate(self, size=None):
        return self.stream.truncate(size)

    def read(self, *args, **kwargs):
        return self.stream.read(*args, **kwargs)

    def readinto(self, *args, **kwargs):
        return self.stream.readinto(*args, **kwargs)

    def write(self, data):
        self.stream.write(data)
        return len(data)
